/*
 * RioloaderMain.java
 * Encoding: ISO-8859-1
 *
 * Copyright (c) 2006 Philipp Frischmuth <philipp@frischmuth24.de>
 *
 * This file is part of pOWL - web based ontology editor.
 *
 * pOWL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * pOWL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with pOWL; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

import java.io.FileInputStream;
import org.openrdf.rio.ParseErrorListener;
import org.openrdf.rio.Parser;
import org.openrdf.rio.rdfxml.RdfXmlParser;

import net.sf.powl.CSVStatementHandler;
import net.sf.powl.POWLParseErrorListener;
import net.sf.powl.POWLStatementHandler;
import net.sf.powl.RAPStatementHandler;

/**
 * This class contains the main method fr the command-line interface.
 * Use this interface as followed: java RioloaderMain <FILENAME> <BASEURI>
 * <POWL | RAP>
 * @author Philipp Frischmuth
 * @version 0.1, 2006-10-25
 */
public class RioloaderMain {

	/**
	 * @param args The arguments, which are needed.
	 */
	public static void main(String[] args) {
		
		// check whether the needed arguments are given
		if (args.length < 3 || !(args[args.length-1].toLowerCase().equals("powl") ||
				args[args.length-1].toLowerCase().equals("rap"))) {
			System.out.println("USAGE: RioloaderMain <FILENAME> <BASEURI>" +
					" <POWL | RAP>");
			return;
		}
		
		String[] files = new String[args.length-2];
		for (int i=0; i<files.length; ++i) {
			files[i] = args[i];
		}
		
		String BASEURI = args[args.length-2];
		String DBLAYOUT = args[args.length-1];
		
		System.setProperty("org.xml.sax.driver",
				"org.apache.xerces.parsers.SAXParser");
		
		Parser parser = new RdfXmlParser();
		ParseErrorListener errorListener = new POWLParseErrorListener();
		CSVStatementHandler statementHandler = null;
		
		if (DBLAYOUT.equals("powl")) {
			statementHandler = new POWLStatementHandler();
		}
		else if (DBLAYOUT.equals("rap")) {
			statementHandler = new RAPStatementHandler();
		}
	
		parser.setStatementHandler(statementHandler);
		parser.setParseErrorListener(errorListener);
		parser.setVerifyData(true);
		parser.setStopAtFirstError(true);
		
		long timeBefore = 0;
		long timeAfter = 0;
			
		try {
			for (int i=0; i<files.length; ++i) {
				timeBefore = System.currentTimeMillis();
				System.out.print("PARSING: " + files[i] + "...");
				parser.parse(new FileInputStream(files[i]), BASEURI);
				timeAfter = System.currentTimeMillis();
				System.out.println("DONE (" + ((timeAfter - timeBefore)/1000.0)
						+ "s)");
			}
			statementHandler.writeDataToFiles();
		} catch (Exception e) {
			e.printStackTrace();
		}

	}
}
