/*
 * RAPStatementHandler.java
 * Encoding: ISO-8859-1
 *
 * Copyright (c) 2006 Philipp Frischmuth <philipp@frischmuth24.de>
 *
 * This file is part of pOWL - web based ontology editor.
 *
 * pOWL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * pOWL is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with pOWL; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package net.sf.powl;

import java.io.FileWriter;
import java.io.IOException;

import org.openrdf.model.Resource;
import org.openrdf.model.URI;
import org.openrdf.model.Value;

/**
 * This class represents an implementation of a StatementHandler, which
 * is used by rio rdf parser.
 * 
 * @author Philipp Frischmuth <philipp@frischmuth24.de>
 * @version 0.1, 2006-10-25
 */
public class RAPStatementHandler implements CSVStatementHandler {

	/*
	 * we do not write to the files each time the handleStatement method runs,
	 * for this would end in a very bad performance... so we write the data to
	 * a string
	 */
	private StringBuilder statements;
	
	/*
	 * we need a FileWriter object in order to write the statements.csv file
	 */
	private FileWriter statementWriter;
	
	/**
	 * The constructor of a RAPStatementHandler object.
	 */
	public RAPStatementHandler() {
		super();
		
		this.statements = new StringBuilder();
		
		// make sure that the needed file exists and that it is empty;
		// this happens only when the object is created, but not each time the
		// handleStatement method runs, for we want to append the new ones to
		// the end of the file
		try {
			this.statementWriter = new FileWriter("rap_statements.csv", false);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * @see CSVStatementHandler
	 */
	public void handleStatement(Resource subject, URI predicate, Value object) {
		
		this.statements.append(subject + "," + predicate + "," +
				object.toString().replace("\n", "").replace(",", "\\,") + "\n");
	}
	
	/**
	 * @see CSVStatementHandler
	 */
	public void writeDataToFiles() {
		
		try {
			this.statementWriter = new FileWriter("rap_statements.csv", true);
			this.statementWriter.append(this.statements);
			this.statementWriter.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
}
